"use strict";
/*!
 * Paradigm UI Web
 * Copyright (c) 2017 Miracle Devs, Inc
 * Licensed under MIT (https://gitlab.com/miracledevs-paradigm/ui-web-angularjs/blob/master/LICENSE)
 */
Object.defineProperty(exports, "__esModule", { value: true });
var angular_1 = require("angular");
var controller_1 = require("./decorators/controller");
var service_1 = require("./decorators/service");
var filter_1 = require("./decorators/filter");
var directive_1 = require("./decorators/directive");
var interceptor_1 = require("./decorators/interceptor");
var paradigm_ui_web_shared_1 = require("@miracledevs/paradigm-ui-web-shared");
var logging_service_1 = require("./services/logging.service");
var angular_service_1 = require("./services/angular.service");
var component_1 = require("./decorators/component");
var ModuleBase = /** @class */ (function () {
    function ModuleBase() {
        var _this = this;
        this.controllersQueue = new Array();
        this.interceptorsQueue = new Array();
        this.module = angular_1.module(this.getModuleName(), this.getModuleDependencies());
        this.register();
        this.module.config([angular_service_1.AngularServices.stateProvider, angular_service_1.AngularServices.urlRouterProvider, angular_service_1.AngularServices.httpProvider, angular_service_1.AngularServices.locationProvider, function (stateProvider, urlRouterProvider, httpProvider, location) { return _this.configureRoutes(stateProvider, urlRouterProvider, httpProvider, location); }]);
        this.module.config([angular_service_1.AngularServices.translateProvider, function (translateProvider) { return _this.configureTranslator(translateProvider); }]);
        this.module.config([angular_service_1.AngularServices.compileProvider, function (compileProvider) { return compileProvider.debugInfoEnabled(false); }]);
        this.module.config([angular_service_1.AngularServices.httpProvider, function (provider) { return _this.registerInterceptorsInternally(provider); }]);
        this.module.config([angular_service_1.AngularServices.stateProvider, function (provider) { return _this.registerStates(provider); }]);
        this.module.run([angular_service_1.AngularServices.rootScope, angular_service_1.AngularServices.state, angular_service_1.AngularServices.injector, function (rootScope, state, injector) { return _this.authorizeRoute(rootScope, state, injector); }]);
    }
    ModuleBase.prototype.getModule = function () {
        return this.module;
    };
    ModuleBase.prototype.getModuleName = function () {
        throw new Error("Please override getModuleName method and provide a name for the module.");
    };
    ModuleBase.prototype.registerControllers = function (controllers) {
        var _this = this;
        paradigm_ui_web_shared_1.ArrayExtensions.forEach(controllers, function (x) { return _this.registerController(x); });
    };
    ModuleBase.prototype.registerController = function (controller) {
        if (paradigm_ui_web_shared_1.ObjectExtensions.isNull(controller) ||
            paradigm_ui_web_shared_1.ObjectExtensions.isNull(controller[controller_1.controllerInfoKey]))
            throw new Error("Problems registering the controller '" + paradigm_ui_web_shared_1.FunctionExtensions.getFunctionName(controller) + "'.");
        var controllerInfo = controller[controller_1.controllerInfoKey];
        if (!paradigm_ui_web_shared_1.ObjectExtensions.isNull(controllerInfo.dependencies))
            controller["$inject"] = controllerInfo.dependencies;
        this.module.controller(controllerInfo.name, controller);
        this.logger.writeMessage("registering " + controllerInfo.name);
        if (this.stateProvider == null)
            this.controllersQueue.push(controller);
        else
            this.registerControllerState(controller);
    };
    ModuleBase.prototype.registerComponents = function (Components) {
        var _this = this;
        paradigm_ui_web_shared_1.ArrayExtensions.forEach(Components, function (x) { return _this.registerComponent(x); });
    };
    ModuleBase.prototype.registerComponent = function (component) {
        if (paradigm_ui_web_shared_1.ObjectExtensions.isNull(component) ||
            paradigm_ui_web_shared_1.ObjectExtensions.isNull(component[component_1.componentInfoKey]))
            throw new Error("Problems registering the component '" + paradigm_ui_web_shared_1.FunctionExtensions.getFunctionName(component) + "'.");
        var componentInfo = component[component_1.componentInfoKey];
        var parameters = (componentInfo.dependencies || new Array());
        parameters.push(componentInfo.component);
        this.module.component(componentInfo.name, {
            controller: parameters,
            controllerAs: componentInfo.componentAs || "component",
            templateUrl: componentInfo.templateUrl,
            template: componentInfo.template,
            bindings: componentInfo.bindings,
            transclude: componentInfo.transclude,
            require: componentInfo.require
        });
        this.logger.writeMessage("registering " + componentInfo.name);
    };
    ModuleBase.prototype.registerServices = function (services) {
        var _this = this;
        paradigm_ui_web_shared_1.ArrayExtensions.forEach(services, function (x) { return _this.registerService(x); });
    };
    ModuleBase.prototype.registerService = function (service) {
        if (paradigm_ui_web_shared_1.ObjectExtensions.isNull(service) ||
            paradigm_ui_web_shared_1.ObjectExtensions.isNull(service[service_1.serviceInfoKey]))
            throw new Error("Problems registering the service '" + paradigm_ui_web_shared_1.FunctionExtensions.getFunctionName(service) + "'.");
        var serviceInfo = service[service_1.serviceInfoKey];
        var parameters = (serviceInfo.dependencies || new Array());
        parameters.push(serviceInfo.factory);
        this.module.factory(serviceInfo.name, parameters);
        this.logger.writeMessage("registering " + serviceInfo.name);
    };
    ModuleBase.prototype.registerLoggingService = function (service) {
        if (paradigm_ui_web_shared_1.ObjectExtensions.isNull(service) ||
            paradigm_ui_web_shared_1.ObjectExtensions.isNull(service[service_1.serviceInfoKey]))
            throw new Error("Problems registering the logging service '" + paradigm_ui_web_shared_1.FunctionExtensions.getFunctionName(service) + "'.");
        var serviceInfo = service[service_1.serviceInfoKey];
        var parameters = (serviceInfo.dependencies || new Array());
        parameters.push(serviceInfo.factory);
        this.module.factory(serviceInfo.name, parameters);
        this.logger = serviceInfo.factory();
        this.logger.writeMessage("registering " + serviceInfo.name);
    };
    ModuleBase.prototype.registerInterceptors = function (interceptors) {
        var _this = this;
        paradigm_ui_web_shared_1.ArrayExtensions.forEach(interceptors, function (x) { return _this.registerInterceptor(x); });
    };
    ModuleBase.prototype.registerInterceptor = function (interceptor) {
        if (paradigm_ui_web_shared_1.ObjectExtensions.isNull(interceptor) ||
            paradigm_ui_web_shared_1.ObjectExtensions.isNull(interceptor[interceptor_1.interceptorInfoKey]))
            throw new Error("Problems registering the interceptor '" + paradigm_ui_web_shared_1.FunctionExtensions.getFunctionName(interceptor) + "'.");
        var interceptorInfo = interceptor[interceptor_1.interceptorInfoKey];
        if (this.httpProvider == null)
            this.interceptorsQueue.push(interceptor);
        else {
            var parameters = (interceptorInfo.dependencies || new Array());
            parameters.push(interceptorInfo.factory);
            this.httpProvider.interceptors.push(parameters);
        }
        this.logger.writeMessage("registering " + interceptorInfo.name);
    };
    ModuleBase.prototype.registerFilters = function (filters) {
        var _this = this;
        paradigm_ui_web_shared_1.ArrayExtensions.forEach(filters, function (x) { return _this.registerFilter(x); });
    };
    ModuleBase.prototype.registerFilter = function (filter) {
        if (paradigm_ui_web_shared_1.ObjectExtensions.isNull(filter) ||
            paradigm_ui_web_shared_1.ObjectExtensions.isNull(filter[filter_1.filterInfoKey]))
            throw new Error("Problems registering the filter '" + paradigm_ui_web_shared_1.FunctionExtensions.getFunctionName(filter) + "'.");
        var filterInfo = filter[filter_1.filterInfoKey];
        var parameters = (filterInfo.dependencies || new Array());
        parameters.push(filterInfo.factory);
        this.module.filter(filterInfo.name, parameters);
        this.logger.writeMessage("registering " + filterInfo.name);
    };
    ModuleBase.prototype.registerDirectives = function (directives) {
        var _this = this;
        paradigm_ui_web_shared_1.ArrayExtensions.forEach(directives, function (x) { return _this.registerDirective(x); });
    };
    ModuleBase.prototype.registerDirective = function (directive) {
        if (paradigm_ui_web_shared_1.ObjectExtensions.isNull(directive) ||
            paradigm_ui_web_shared_1.ObjectExtensions.isNull(directive[directive_1.directiveInfoKey]))
            throw new Error("Problems registering the directive '" + paradigm_ui_web_shared_1.FunctionExtensions.getFunctionName(directive) + "'.");
        var directiveInfo = directive[directive_1.directiveInfoKey];
        var parameters = (directiveInfo.dependencies || new Array());
        parameters.push(directiveInfo.factory);
        this.module.directive(directiveInfo.name, parameters);
        this.logger.writeMessage("registering " + directiveInfo.name);
    };
    ModuleBase.prototype.registerStates = function (stateProvider) {
        this.stateProvider = stateProvider;
        if (!paradigm_ui_web_shared_1.ObjectExtensions.isNull(this.controllersQueue)) {
            for (var i = 0; i < this.controllersQueue.length; i++) {
                this.registerControllerState(this.controllersQueue[i]);
            }
            this.controllersQueue = null;
        }
    };
    ModuleBase.prototype.registerInterceptorsInternally = function (httpProvider) {
        this.httpProvider = httpProvider;
        if (!paradigm_ui_web_shared_1.ObjectExtensions.isNull(this.interceptorsQueue)) {
            for (var i = 0; i < this.interceptorsQueue.length; i++) {
                this.registerInterceptor(this.interceptorsQueue[i]);
            }
            this.interceptorsQueue = null;
        }
    };
    ModuleBase.prototype.registerControllerState = function (controller) {
        var controllerInfo = controller[controller_1.controllerInfoKey];
        if (controllerInfo.stateName == null)
            return;
        this.stateProvider.state(controllerInfo.stateName, {
            url: controllerInfo.stateUrl,
            templateUrl: controllerInfo.templateUrl,
            resolve: controllerInfo.resolve
        });
    };
    ModuleBase.prototype.configureRoutes = function (stateProvider, urlRouterProvider, httpProvider, locationProvider) {
    };
    ModuleBase.prototype.authorizeRoute = function (rootScope, state, injector) {
    };
    ModuleBase.prototype.configureTranslator = function (translateProvider) {
    };
    ModuleBase.prototype.getModuleDependencies = function () {
        throw new Error("Please override getModuleDependencies method and provide other required modules or an empty array.");
    };
    ModuleBase.prototype.register = function () {
        // override this method to change the base logging service.
        this.registerLoggingService(logging_service_1.DummyLoggingService);
    };
    return ModuleBase;
}());
exports.ModuleBase = ModuleBase;
