"use strict";
/*!
 * Paradigm UI Web
 * Copyright (c) 2017 Miracle Devs, Inc
 * Licensed under MIT (https://gitlab.com/miracledevs-paradigm/ui-web-angularjs/blob/master/LICENSE)
 */
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
var service_1 = require("../decorators/service");
var framework_service_1 = require("./framework.service");
var base_service_1 = require("./base.service");
var message_bus_service_interface_1 = require("./message-bus.service.interface");
var paradigm_ui_web_shared_1 = require("@miracledevs/paradigm-ui-web-shared");
var message_1 = require("../decorators/message");
var MessageBus = /** @class */ (function (_super) {
    __extends(MessageBus, _super);
    function MessageBus() {
        var _this = _super.call(this) || this;
        _this.handlers = new paradigm_ui_web_shared_1.Dictionary();
        return _this;
    }
    MessageBus_1 = MessageBus;
    MessageBus.prototype.count = function () {
        return this.handlers.count();
    };
    MessageBus.prototype.isRegistered = function (messageType) {
        var type = messageType[message_1.messageTypeKey];
        return this.handlers.containsKey(type);
    };
    MessageBus.prototype.handlerCount = function (messageType) {
        var type = messageType[message_1.messageTypeKey];
        if (!this.handlers.containsKey(type))
            return 0;
        return this.handlers.get(type).count();
    };
    MessageBus.prototype.register = function (messageType, handler) {
        var type = messageType[message_1.messageTypeKey];
        if (!this.handlers.containsKey(type))
            this.handlers.add(type, new paradigm_ui_web_shared_1.ArrayList());
        var messageBusHandler = new message_bus_service_interface_1.MessageBusHandler(messageType, handler);
        this.handlers.get(type).add(messageBusHandler);
        return messageBusHandler.token;
    };
    MessageBus.prototype.unregister = function (token) {
        var type = token.type[message_1.messageTypeKey];
        if (!this.handlers.containsKey(type))
            return;
        var handler = this.handlers.get(type);
        handler.removeAll(function (x) { return x.token.guid === token.guid; });
        if (!handler.any())
            this.handlers.remove(type);
    };
    MessageBus.prototype.send = function (message, token) {
        if (paradigm_ui_web_shared_1.ObjectExtensions.isNull(message))
            throw new Error("Can not send a null message.");
        if (paradigm_ui_web_shared_1.ObjectExtensions.isNull(message.constructor))
            throw new Error("Message does not have a constructor, and the system can not infer the type.");
        var type = message.constructor[message_1.messageTypeKey];
        if (!this.handlers.containsKey(type))
            return false;
        var handlers = this.handlers.get(type)
            .where(function (x) { return token == null || token.guid === x.token.guid; })
            .select(function (x) { return x.handler; });
        var accepted = false;
        for (var i = (handlers.count() - 1); i >= 0; i--) {
            var handler = handlers.get(i);
            if (handler == null)
                continue;
            accepted = true;
            handler(message);
        }
        return accepted;
    };
    MessageBus.factory = function () {
        return new MessageBus_1();
    };
    MessageBus = MessageBus_1 = __decorate([
        service_1.Service({
            name: framework_service_1.FrameworkServices.messageBus,
            factory: MessageBus_1.factory
        })
    ], MessageBus);
    return MessageBus;
    var MessageBus_1;
}(base_service_1.ServiceBase));
exports.MessageBus = MessageBus;
